'' =================================================================================================
''
''   File....... rgbled.spin 
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 04 MAR 2014
''
'' =================================================================================================
''
'' Simple RGB LED controller in PASM
''
'' Requires pointer to four consecutive longs
''
''  -- rgb color in form: $00_RR_GG_BB
''  -- red pin
''  -- green pin
''  -- blue pin


pub start(p_rgbcolor)

'' Start RGB cog

  cognew(@rgbcog, p_rgbcolor)
  

dat

                        org     0

rgbcog                  mov     r1, par                         ' address of parameters

                        add     r1, #4                          ' skip color (in par)
                        rdlong  r2, r1                          ' read read pin
                        mov     rmask, #1                       ' convert to mask
                        shl     rmask, r2
                        or      dira, rmask                     ' make output

                        add     r1, #4
                        rdlong  r2, r1
                        mov     gmask, #1
                        shl     gmask, r2
                        or      dira, gmask

                        add     r1, #4
                        rdlong  r2, r1
                        mov     bmask, #1
                        shl     bmask, r2
                        or      dira, bmask

                        rdlong  ltix, #0
                        shr     ltix, #7                        ' tix for 128Hz
                        shr     ltix, #8                        ' tix for 256 steps

                        mov     ltimer, ltix
                        add     ltimer, cnt                     ' start loop timer

                        mov     cycle, #0

rgbloop                 rdlong  r1, par                         ' get color, 0x00_RR_GG_BB

do_red                  mov     r2, r1                          ' make copy
                        shr     r2, #16                         ' isolate red 
                        and     r2, #$FF                wz
        if_z            andn    outa, rmask
        if_z            jmp     #do_green
                        cmp     cycle, r2               wc, wz  ' check value against cycle
        if_be           or      outa, rmask
        if_a            andn    outa, rmask

do_green                mov     r2, r1
                        shr     r2, #8                          ' isolate green                                                                   
                        and     r2, #$FF                wz
        if_z            andn    outa, gmask
        if_z            jmp     #do_blue                                                                      
                        cmp     cycle, r2               wc, wz  
        if_be           or      outa, gmask                     
        if_a            andn    outa, gmask

do_blue                 mov     r2, r1                          
                        and     r2, #$FF                wz      ' isolate blue
        if_z            andn    outa, bmask
        if_z            jmp     #next_cycle                                                                                                       
                        cmp     cycle, r2               wc, wz  
        if_be           or      outa, bmask                     
        if_a            andn    outa, bmask

next_cycle              add     cycle, #1                       ' bump cycle
                        and     cycle, #$FF                     ' keep 0..255

                        waitcnt ltimer, ltix                    ' let loop timing expire
                        jmp     #rgbloop                        

' ---------------------------------------------------------------------------------------

rmask                   res     1                               ' output masks
gmask                   res     1
bmask                   res     1

rgbhub                  res     1                               ' hub address or RGB value

ltix                    res     1                               ' loop tix for ~1kHz freq
ltimer                  res     1                               ' loop timer

cycle                   res     1                               ' pwm cycle position

r1                      res     1                               ' working registers
r2                      res     1
                         
                        fit     496                              